@@ -5,6 +5,7 @@ require 'rspec'
 require 'shellwords'
 require 'pg'
 require 'openssl'
+require 'fileutils'
 require 'objspace'
 require_relative 'helpers/scheduler.rb'
 require_relative 'helpers/tcp_gate_scheduler.rb'
@@ -15,6 +16,13 @@ TEST_DIR_STR = ENV['RUBY_PG_TEST_DIR'] || DEFAULT_TEST_DIR_STR
 TEST_DIRECTORY = Pathname.new(TEST_DIR_STR)
 DATA_OBJ_MEMSIZE = ObjectSpace.memsize_of(Object.new)
 
+if defined? Ractor
+	class Ractor
+		# compat with Ruby-3.4 and older
+		alias value take unless method_defined? :value
+	end
+end
+
 module PG::TestingHelpers
 
 	### Automatically wrap a transaction around examples that don't disable it.
@@ -250,7 +258,7 @@ module PG::TestingHelpers
 		attr_reader :pgdata
 
 		### Set up a PostgreSQL database instance for testing.
-		def initialize( name, port: 54321, postgresql_conf: '' )
+		def initialize( name, port: 23456, postgresql_conf: '' )
 			trace "Setting up test database for #{name}"
 			@name = name
 			@port = port
@@ -475,6 +483,16 @@ EOT
 		end
 	end
 
+	class ListenSocket
+		attr_reader :port
+		def initialize(host = '127.0.0.1')
+			TCPServer.open( host, 0 ) do |serv|
+				@port = serv.local_address.ip_port
+				yield self
+			end
+		end
+	end
+
 	def check_for_lingering_connections( conn )
 		conn.exec( "SELECT * FROM pg_stat_activity" ) do |res|
 			conns = res.find_all {|row| row['pid'].to_i != conn.backend_pid && ["client backend", nil].include?(row["backend_type"]) }
@@ -594,7 +612,7 @@ EOT
 		# Run examples with gated scheduler
 		sched = Helpers::TcpGateScheduler.new(external_host: 'localhost', external_port: ENV['PGPORT'].to_i, debug: ENV['PG_DEBUG']=='1')
 		Fiber.set_scheduler(sched)
-		@conninfo_gate = @conninfo.gsub(/(^| )port=\d+/, " port=#{sched.internal_port} sslmode=disable")
+		@conninfo_gate = @conninfo.gsub(/(^| )port=\d+/, " port=#{sched.internal_port}")
 
 		# Run examples with default scheduler
 		#Fiber.set_scheduler(Helpers::Scheduler.new)
@@ -640,7 +658,7 @@ EOT
 	def gate_setup
 		# Run examples with gate
 		gate = Helpers::TcpGateSwitcher.new(external_host: 'localhost', external_port: ENV['PGPORT'].to_i, debug: ENV['PG_DEBUG']=='1')
-		@conninfo_gate = @conninfo.gsub(/(^| )port=\d+/, " port=#{gate.internal_port} sslmode=disable")
+		@conninfo_gate = @conninfo.gsub(/(^| )port=\d+/, " port=#{gate.internal_port}")
 
 		# Run examples without gate
 		#@conninfo_gate = @conninfo
@@ -700,12 +718,9 @@ RSpec.configure do |config|
 		config.filter_run_excluding :windows
 	end
 
-	config.filter_run_excluding( :postgresql_94 ) if PG.library_version <  90400
-	config.filter_run_excluding( :postgresql_95 ) if PG.library_version <  90500
-	config.filter_run_excluding( :postgresql_96 ) if PG.library_version <  90600
-	config.filter_run_excluding( :postgresql_10 ) if PG.library_version < 100000
 	config.filter_run_excluding( :postgresql_12 ) if PG.library_version < 120000
 	config.filter_run_excluding( :postgresql_14 ) if PG.library_version < 140000
+	config.filter_run_excluding( :postgresql_17 ) if PG.library_version < 170000
 	config.filter_run_excluding( :unix_socket ) if RUBY_PLATFORM=~/mingw|mswin/i
 	config.filter_run_excluding( :scheduler ) if RUBY_VERSION < "3.0" || (RUBY_PLATFORM =~ /mingw|mswin/ && RUBY_VERSION < "3.1") || !Fiber.respond_to?(:scheduler)
 	config.filter_run_excluding( :scheduler_address_resolve ) if RUBY_VERSION < "3.1"
@@ -722,7 +737,7 @@ RSpec.configure do |config|
 		PG::TestingHelpers.stop_existing_postmasters
 
 		ENV['PGHOST'] = 'localhost'
-		ENV['PGPORT'] ||= "54321"
+		ENV['PGPORT'] ||= "23456"
 		port = ENV['PGPORT'].to_i
 		$pg_server = PG::TestingHelpers::PostgresServer.new("specs", port: port)
 		$pg_server.create_test_db
@@ -731,3 +746,12 @@ RSpec.configure do |config|
 		$pg_server.teardown
 	end
 end
+
+
+# Do not wait for threads doing blocking calls at the process shutdown.
+# Instead exit immediately after printing the rspec report, if we know there are pending IO calls, which do not react on ruby interrupts.
+END{
+	if $scheduler_timeout
+		exit!(1)
+	end
+}
